package org.xbib.interlibrary.http.server;

import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.DomContentJoiner;
import org.xbib.j2html.tags.Tag;
import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.UserProfile;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.util.Locale;
import java.util.Objects;

import static org.xbib.j2html.TagCreator.a;
import static org.xbib.j2html.TagCreator.abbr;
import static org.xbib.j2html.TagCreator.body;
import static org.xbib.j2html.TagCreator.button;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.document;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.footer;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.head;
import static org.xbib.j2html.TagCreator.header;
import static org.xbib.j2html.TagCreator.hr;
import static org.xbib.j2html.TagCreator.html;
import static org.xbib.j2html.TagCreator.i;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.li;
import static org.xbib.j2html.TagCreator.link;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.meta;
import static org.xbib.j2html.TagCreator.nav;
import static org.xbib.j2html.TagCreator.rawHtml;
import static org.xbib.j2html.TagCreator.script;
import static org.xbib.j2html.TagCreator.span;
import static org.xbib.j2html.TagCreator.title;
import static org.xbib.j2html.TagCreator.ul;

public class BaseView {

    protected final HtmlTemplateResource resource;

    protected final Application application;

    protected final HttpRequest request;

    protected final String search;

    protected String theme;

    protected String inverseTheme;

    public BaseView(HtmlTemplateResource resource,
                    Application application,
                    HttpRequest request) throws ParameterException {
        this.resource = Objects.requireNonNull(resource, "resource must not be null");
        this.application = Objects.requireNonNull(application, "application must not be null");
        this.request = Objects.requireNonNull(request, "requets must not be null");

        // nav bar search
        this.search = request.getParameter().getAsString("query", Parameter.Domain.QUERY);
    }

    public String createSimplePage(HttpRouterContext context,
                                   String pageTitle, Tag<?>... tags) {
        return document(
                html().attr("lang", "de").with(
                        head(
                                title(pageTitle),
                                headContent()
                        ),
                        body().with(
                                simpleNavContent(),
                                headerContent(),
                                mainContent(context, tags),
                                simpleFooterContent()
                        )
                )
        );
    }

    public String createPage(HttpRouterContext context,
                             String pageTitle, Tag<?>... tags) {
        UserProfile userProfile = context.getAttributes().get(UserProfile.class, "userprofile");
        if (userProfile == null) {
            // emergency exit, need to show login, but no security domain is implemented
            return document(html().with(body().with(main("User profile went off-line and page can not be created. This is an implementation error. No login resource implemented."))));
        }
        theme = userProfile.getAttributes().get(String.class, "theme");
        inverseTheme = "dark".equals(theme) ? "light" : "dark";
        return document(
                html().attr("lang", "de").attr("data-bs-theme", theme).with(
                        head(
                                title(pageTitle),
                                headContent()
                        ),
                        body().with(
                                navContent(context),
                                headerContent(),
                                mainContent(context, tags),
                                footerContent(),
                                script().with(themeScript("/action/theme/"))
                        )
                )
        );
    }

    protected DomContent headContent() {
        return DomContentJoiner.join("", true,
                meta().withCharset("utf-8"),
                meta().attr("name", "viewport")
                        .attr("content", "width=device-width, initial-scale=1.0"),
                link().attr("rel", "icon")
                        .attr("type", "image/x-icon")
                        .attr("href", resource.contextPath(request,"favicon.ico")),
                scriptContent()
        );
    }

    protected DomContent scriptContent() {
        return DomContentJoiner.join("", true,
                link().attr("rel", "stylesheet").attr("href", bootstrapCss()),
                link().attr("rel", "stylesheet").attr("href", bootstrapIconsCss()),
                link().attr("rel", "stylesheet").attr("href", bootstrapNotifyCss()),
                link().attr("rel", "stylesheet").attr("href", localCss()),
                script().attr("src", bootstrapJs()),
                script().attr("src", bootstrapNotifyJs()),
                script().attr("src", localJs())
        );
    }

    protected DomContent headerContent() {
        return header().attr("class", "header").withText("");
    }

    protected DomContent simpleNavContent() {
        return nav().attr("class", "navbar navbar-light bg-light navbar-expand-lg bg-body-tertiary").with(
                div().attr("class", "container").with(
                        div().attr("class", "navbar-brand").with(
                                rawHtml(hbzLogo())
                        )
                )
        );
    }

    protected DomContent navContent(HttpRouterContext context) {
        UserProfile userProfile = Objects.requireNonNull(context.getAttributes().get(UserProfile.class, "userprofile"));
        String effectiveUserId = userProfile.getEffectiveUserId();
        boolean isAdmin = context.isAuthenticated() && userProfile.getRoles().contains("Admin");
        String backButtonUrl = String.format(application.getSettings().get("web.nav.backbutton.url"), effectiveUserId);
        String backButtonLabel = application.getSettings().get("web.nav.backbutton.label");

        String zflUrl = String.format(application.getSettings().get("web.nav.zfl.url"), effectiveUserId);
        String zflLabel = application.getSettings().get("web.nav.zfl.label");

        String zflAflUrl = String.format(application.getSettings().get("web.nav.zfl.afl.url"), effectiveUserId);
        String zflAflLabel = application.getSettings().get("web.nav.zfl.afl.label");
        String zflPflUrl = String.format(application.getSettings().get("web.nav.zfl.pfl.url"), effectiveUserId);
        String zflPflLabel = application.getSettings().get("web.nav.zfl.pfl.label");

        String medeaAflUrl = String.format(application.getSettings().get("web.nav.medea.afl.url"), effectiveUserId);
        String medeaAflLabel = application.getSettings().get("web.nav.medea.afl.label");
        String medeaPflUrl = String.format(application.getSettings().get("web.nav.medea.pfl.url"), effectiveUserId);
        String medeaPflLabel = application.getSettings().get("web.nav.medea.pfl.label");

        return nav().attr("class", "navbar navbar-" + theme + " bg-" + theme + " navbar-expand-lg bg-body-tertiary").with(
                div().attr("class", "container").with(
                        a().attr("class", "navbar-brand")
                                .attr("title", application.getSettings().get("web.nav.brand.label"))
                                .attr("href", application.getSettings().get("web.nav.brand.url")).with(
                                        rawHtml(hbzLogo("width: auto", "92", "32"))
                                ),
                        div().attr("class", "navbar-header")
                                .attr("style", "font-size: larger")
                                .withText("Aufsätze"),
                        div().attr("id", "nav")
                                .attr("class", "collapse navbar-collapse").with(
                                        ul().attr("class", "navbar-nav me-auto mb-2 mb-lg-0").with(
                                                li().attr("class", "nav-item dropdown").with(
                                                        a().attr("class", "nav-link dropdown-toggle")
                                                                .attr("role", "button")
                                                                .attr("data-bs-toggle", "dropdown")
                                                                .attr("aria-expanded", "false")
                                                                .attr("href", "#")
                                                                .withText("Navigation"),
                                                        ul().attr("class", "dropdown-menu").with(
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Aufsätze im ZFL")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl")
                                                                                .withText("Einstiegsseite")
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Monografien im ZFL")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", zflUrl)
                                                                                .withText(zflLabel)
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", zflAflUrl)
                                                                                .withText(zflAflLabel)
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", zflPflUrl)
                                                                                .withText(zflPflLabel)
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Medea")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", medeaAflUrl)
                                                                                .withText(medeaAflLabel)
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", medeaPflUrl)
                                                                                .withText(medeaPflLabel)
                                                                )
                                                        )
                                                ),
                                                li().attr("class", "nav-item dropdown").with(
                                                        a().attr("class", "nav-link dropdown-toggle")
                                                                .attr("role", "button")
                                                                .attr("data-bs-toggle", "dropdown")
                                                                .attr("aria-expanded", "false")
                                                                .attr("href", "#")
                                                                .withText("Passiver Leihverkehr"),
                                                        ul().attr("class", "dropdown-menu").with(
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Aufsätze im ZFL")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/form/new")
                                                                                .withText("Neue Bestellung erfassen")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/order/list/passive/")
                                                                                .withText("Bestellungen anzeigen")
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Aufsätze in Medea")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/search/")
                                                                                .withText("Katalogsuche mit Bestellfunktion")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/form/free")
                                                                                .withText("Freie Bestellung")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/medea/list/passive/")
                                                                                .withText("Bestellung anzeigen")
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/delivery/list/passive/")
                                                                                .withText("Eingehende Lieferungen")
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Statistik")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", resource.contextPath(request, "royalties"))
                                                                                .withText("VG WORT Meldekontrolle")
                                                                )
                                                        )
                                                ),
                                                li().attr("class", "nav-item dropdown").with(
                                                        a().attr("class", "nav-link dropdown-toggle")
                                                                .attr("role", "button")
                                                                .attr("data-bs-toggle", "dropdown")
                                                                .attr("aria-expanded", "false")
                                                                .attr("href", "#")
                                                                .withText("Aktiver Leihverkehr"),
                                                        ul().attr("class", "dropdown-menu").with(
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Aufsätze im ZFL")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/order/list/active/")
                                                                                .withText("Lieferungen anzeigen")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/delivery/list/active/")
                                                                                .withText("Abgesetzte Lieferungen anzeigen")
                                                                ),
                                                                li().with(
                                                                        hr().attr("class", "dropdown-divider")
                                                                ),
                                                                li().with(
                                                                        div().attr("class", "dropdown-header")
                                                                                .withText( "Aufsätze in Medea")
                                                                ),
                                                                li().with(
                                                                        a().attr("class", "dropdown-item")
                                                                                .attr("href", "/app/fl/medea/list/active/")
                                                                                .withText("Lieferungen anzeigen")
                                                                )
                                                        )
                                                ),
                                        form().attr("class", "d-flex")
                                                .attr("role", "search")
                                                .attr("method", "get")
                                                .attr("action", "/app/fl/order")
                                                .with(
                                                        input().attr("class", "form-control me-2")
                                                                .attr("type", "search")
                                                                .attr("placeholder", "Suche")
                                                                .attr("aria-label", "Suche")
                                                                .attr("name", "search")
                                                                .attr("value", search),
                                                        button().attr("class", "btn btn-outline-primary")
                                                                .attr("type", "submit")
                                                                .with(i().attr("class", "bi bi-search"))
                                                )
                                        )
                                ),
                        div().attr("class", "navbar-header").with(
                                div().attr("class", "input-group input-group-btn").with(
                                        button().attr("class", "btn btn-outline-" + inverseTheme)
                                                .attr("type", "button")
                                                .attr("onclick", "location.assign('" + backButtonUrl + "');return false;").with(
                                                        i().attr("class", "bi bi-box-arrow-left"),
                                                        span().withText(" " + backButtonLabel)
                                                )
                                )
                        ),
                        ul().attr("class", "navbar-nav me-auto mb-2 mb-lg-0").with(
                             li().attr("class", "nav-item dropdown").with(
                                     a().attr("class", "nav-link dropdown-toggle")
                                             .attr("role", "button")
                                             .attr("data-bs-toggle", "dropdown")
                                             .attr("aria-expanded", "false")
                                             .attr("href", "#")
                                             .with(span().with(isAdmin ?
                                                         i().attr("class", "bi bi-person-circle").attr("style", "color: darkorange") :
                                                         i().attr("class", "bi bi-person-circle"),
                                                 span().withText(" " + userProfile.getEffectiveUserId() + " "))),
                                     ul().attr("class", "dropdown-menu").with(
                                             li().with(
                                                     div().attr("class", "dropdown-header")
                                                             .withText( "Profileinstellungen")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("title", "")
                                                             .attr("href", "#")
                                                             .attr("onclick", "setTheme('dark');return false;")
                                                             .withText("Auf dunkel stellen")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("title", "")
                                                             .attr("href", "#")
                                                             .attr("onclick", "setTheme('light');return false;")
                                                             .withText("Auf hell stellen")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("title", "")
                                                             .attr("href", "#")
                                                             .attr("onclick", "setTheme('auto');return false;")
                                                             .withText("Auf Systemvorgabe stellen")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Konfiguration")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Passwort ändern")
                                             ),
                                             li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", resource.contextPath(request, "logout"))
                                                             .withText("Abmelden")
                                             ),
                                             isAdmin ? li().with(
                                                     hr().attr("class", "dropdown-divider")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Administration")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Status")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", resource.contextPath(request, "admin/userswitch"))
                                                             .withText("Bibliothek wechseln")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Alle Bestellungen anzeigen")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Neue Bibliothek")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Gestrandete Dateien")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Statistik")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Ereignisse")
                                             ) : null,
                                             isAdmin ? li().with(
                                                     a().attr("class", "dropdown-item")
                                                             .attr("href", "#")
                                                             .withText("Leitweganalyse")
                                             ) : null,
                                             null
                                     )
                             )
                        )
                )
        );
    }

    protected DomContent themeScript(String themeUrl) {
        return rawHtml("""
/*<![CDATA[*/

function setTheme(theme) {
  var oldTheme = '%s';
  if (!theme || theme === 'null' || theme === 'auto') {
    theme = "dark";
    if (window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches) {
       theme = "dark";
    } else {
       theme = "light";
    }
  }
  if (theme != oldTheme) {
    fetch("%s" + theme, {
      method: 'get'
    }).then((response) => {
      if (response.status === 200) {
        // only if theme changed, reload
        if (theme != oldTheme) {
          window.location.reload();
        }
      }
      return response.json()
    }).then((res) => {
    }).catch((error) => {
       console.log(error)
    })
  }
}

/* page load complete */
document.addEventListener("DOMContentLoaded", function () {
  // set theme only when there is no theme set
  if ('%s' === 'null') {
    setTheme("auto");
  }
});

/*]]>*/
                """.formatted(this.theme, resource.contextPath(request, themeUrl), this.theme));
    }

    protected String hbzLogo() {
        return hbzLogo("width: auto; height:auto", "185", "65");
    }

    protected String hbzLogo(String style, String width, String height) {
        return String.format("""
                <svg style="%s" viewBox="0 0 185 65", width="%s" height="%s">
                  <path d="M185.191 54.057v-7.596h-5.649c-2.402 0-5.973 0-7.336.065l12.726-18.829V21.79h-22.596v7.401h5.713c3.117 0 5.65-.064 6.883-.13L161.946 48.15v5.908h23.245M94.26 13.024v41.033h8.31V38.475c0-5.649 3.247-10.453 7.467-10.453 3.116 0 4.025 2.013 4.025 5.389v20.646h8.247V31.398c0-5.649-2.337-10.323-9.61-10.323-5.065 0-8.766 2.402-10.648 6.038l-.065-.065c.454-1.883.585-4.61.585-6.558V6.596c-4.805-.13-9.806.065-13.96.585v5.193c1.882.13 4.09.39 5.648.65" fill="#f18123"></path>
                  <path d="M137.622 6.503c-4.805-.13-9.804.065-13.96.584v5.194c1.883.13 4.09.39 5.65.65v40.125c3.7 1.039 7.336 1.558 12.011 1.558 11.103 0 17.466-7.532 17.466-18.44 0-9.544-4.155-15.128-11.882-15.128-4.545 0-8.116 2.208-9.74 5.714h-.064c.324-1.688.519-4.35.519-6.298V6.503zm12.402 30.515c0 7.987-3.377 10.973-8.117 10.973-1.753 0-3.116-.194-4.285-.584v-9.09c0-5.389 2.597-10.388 6.882-10.388 3.766 0 5.52 3.051 5.52 9.09" fill="#f18123" fill-rule="evenodd"></path>
                  <path d="M50.832 42.772V11.641H13.903V5.625l49.105-.001.003 37.148H50.83" fill="#004678"></path>
                  <path d="M21.654 44.5h43.084V5.636l7.428.001-.001 48.399h-50.51V44.5" fill="#004678"></path>
                  <path d="M19.928 13.368v42.395h43.083l-.002 9.763H13.903V13.368h6.025" fill="#004678"></path>
                  <path d="M63.008 3.897H12.177v50.139H0V.002L63.008 0v3.897" fill="#004678"></path>
                </svg>
                """, style, width, height);
    }


    protected DomContent mainContent(HttpRouterContext context, Tag<?>... tags) {
        return main(tags);
    }

    protected DomContent simpleFooterContent() {
        return footer().attr("class", "footer bg-light").with(
                div().attr("class", "container small").with(
                        footerDiv()
                )
        );
    }

    protected DomContent footerContent() {
        return footer().attr("class", "footer bg-" + theme).with(
                div().attr("class", "container small").with(
                        footerDiv(),
                        div().attr("class", "col-md-12").with(each(application.getAttributes(), entry ->
                                span().attr("class", "small invisible")
                                        .withText(entry.getKey().toUpperCase(Locale.ROOT) + ": " + entry.getValue() + " ")))
                )
        );
    }

    private DomContent footerDiv() {
        return div().attr("class", "row").with(
                div().attr("class", "col-md-12").with(
                        abbr().attr("title", "Copyright").withText("©"),
                        span("2024 Hochschulbibliothekszentrum des Landes Nordrhein-Westfalen (hbz)")
                ),
                div().attr("class", "col-md-12").with(
                        span("Jülicher Straße 6 | 50674 Köln | "),
                        a().attr("href", "mailto:fernleihe@hbz-nrw.de")
                                .withText("fernleihe@hbz-nrw.de")
                ),
                div().attr("class", "col-md-12").withText(
                        "Ein Dienst der Leihverkehrszentrale NRW in Kooperation mit den wissenschaftlichen Hochschulbibliotheken und " +
                                "öffentlichen Bibliotheken in Nordrhein-Westfalen und " +
                                "wissenschaftlichen Bibliotheken in Rheinland-Pfalz")
        );
    }

    public String localCss() {
        return resource.contextPath(request, "resources/dist/css/local.css");
    }

    public String localJs() {
        return resource.contextPath(request, "resources/dist/js/local.js");
    }

    public String bootstrapCss() {
        return resource.contextPath(request, "resources/webjars/bootstrap/5.3.3/css/bootstrap.min.css");
    }

    public String bootstrapJs() {
        return resource.contextPath(request, "resources/webjars/bootstrap/5.3.3/js/bootstrap.bundle.min.js");
    }

    public String jqueryJs() {
        return resource.contextPath(request, "resources/webjars/jquery/3.7.1/jquery.min.js");
    }

    public String bootstrapIconsCss() {
        return resource.contextPath(request, "resources/webjars/bootstrap-icons/1.11.3/font/bootstrap-icons.css");
    }

    public String bootstrapNotifyCss() {
        return resource.contextPath(request, "resources/dist/css/bootstrap-notify.css");
    }

    public String bootstrapNotifyJs() {
        return resource.contextPath(request, "resources/dist/js/bootstrap-notify.js");
    }
}
