package org.xbib.interlibrary.http.server;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.ServiceLoader;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.config.ConfigLoader;
import org.xbib.config.ConfigParams;
import org.xbib.event.common.EventManager;
import org.xbib.interlibrary.common.Interlibrary;
import org.xbib.net.NetworkClass;
import org.xbib.net.http.HttpAddress;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.application.ApplicationModule;
import org.xbib.net.http.server.domain.BaseHttpDomain;
import org.xbib.net.http.server.domain.BaseHttpDomainBuilder;
import org.xbib.net.http.server.domain.HttpDomain;
import org.xbib.net.http.server.domain.HttpSecurityDomain;
import org.xbib.net.http.server.executor.BaseExecutor;
import org.xbib.net.http.server.netty.NettyHttpServer;
import org.xbib.net.http.server.netty.NettyHttpServerConfig;
import org.xbib.net.http.server.route.BaseHttpRouter;
import org.xbib.net.http.server.route.BaseHttpRouterBuilder;
import org.xbib.net.http.server.route.HttpRouter;
import org.xbib.settings.Settings;

public class InterlibraryServer extends Interlibrary {

    private static final Logger logger = Logger.getLogger(InterlibraryServer.class.getName());

    private static final boolean debug;

    private final List<InterlibraryServerModule> interlibraryServerModules;

    private EventManager eventManager;

    static {
        debug = "true".equals(System.getProperty("application.debug"));
    }

    public static void main(String[] args) {
        int rc = 0;
        InterlibraryServer server = null;
        if (debug) {
            logger.log(Level.INFO, Arrays.asList(args).toString());
            logger.log(Level.INFO, System.getProperties().toString());
        }
        try {
            // InterlibraryServer server = new InterlibraryServer() does not work for our purposes.
            // Hack: we find the main class class from command line
            // and instantiate from there to activate the overridden methods
            // In that way, the "main" static method is somehow "inherited".
            // The main class name is fortunately present in the property sun.java.command
            String mainClassNameWithArgs = Objects.requireNonNull(System.getProperty("sun.java.command"),
                    "sun.java.command property not present in this JVM");
            // truncate sun java.command if there are arguments
            String mainClassName = mainClassNameWithArgs.contains(" ") ?
                mainClassNameWithArgs.substring(0, mainClassNameWithArgs.indexOf(" ")) : mainClassNameWithArgs;
            // loading class "again" - how could we reference it without loading and instantiating?
            ClassLoader classLoader = InterlibraryServer.class.getClassLoader();
            Class<?> mainClass = classLoader.loadClass(mainClassName);
            logger.log(Level.INFO, "booting main class " + mainClass.getName());
            server = (InterlibraryServer) mainClass.getDeclaredConstructor().newInstance();
            final InterlibraryServer interlibraryServer = server;
            Runtime.getRuntime().addShutdownHook(new Thread(() -> {
                final Logger shutdownLogger = Logger.getLogger("shutdown");
                try {
                    shutdownLogger.log(Level.INFO, "before closing interlibrary server");
                    interlibraryServer.close();
                    shutdownLogger.log(Level.INFO, "after closing interlibrary server");
                } catch (Exception e) {
                    shutdownLogger.log(Level.SEVERE, e.getMessage(), e);
                }
            }));
            String name = System.getProperty("application.name");
            String profile = System.getProperty("application.profile");
            ConfigParams configParams = new ConfigParams()
                    .withArgs(args)
                    .withDirectoryName(name)
                    .withFileNamesWithoutSuffix(profile);
            ConfigLoader configLoader = ConfigLoader.getInstance();
            server.open(classLoader,
                    configLoader, configParams);
            server.run();
            rc = server.getReturnCode();
            logger.log(Level.INFO,  server + " completed, return code = " + rc);
            if (rc == 0) {
                logger.log(Level.INFO, "closing " + server);
                server.close();
                rc = server.getReturnCode();
                logger.log(Level.INFO, "closed: " + server + " return code = " + rc);
            }
        } catch (Throwable t) {
            logger.log(Level.SEVERE, "unexpected exception, setting JVM exit code to 1: " + t.getMessage(), t);
            try {
                if (server != null) {
                    server.close();
                }
            } catch (Exception e) {
               logger.log(Level.SEVERE, e.getMessage(), e);
            }
            rc = 1;
        } finally {
            // always hard-exit the JVM, maybe there are threads hanging
            System.exit(rc);
        }
    }

    public InterlibraryServer() {
        this.interlibraryServerModules = new ArrayList<>();
    }

    @Override
    public void run() {
        super.run();
        HttpAddress httpAddress = buildHttpAddress();
        Settings settings = getSettings();
        String serverName = settings.get("servername", "localhost");
        String serverVendor = settings.get("servervendor", "Java");
        NettyHttpServerConfig serverConfig = new NettyHttpServerConfig();
        serverConfig.setServerName(serverName, serverVendor);
        String networkClassSetting = settings.get("networkclass", "local");
        NetworkClass networkClass = NetworkClass.valueOf(networkClassSetting.toUpperCase(Locale.ROOT));
        serverConfig.setNetworkClass(networkClass);
        serverConfig.setDebug(settings.getAsBoolean("debug", false));
        String secret = getSettings().get("session.secret", null);
        HttpSecurityDomain securityDomain = buildSecurityDomain();
        try (NettyHttpServer server = NettyHttpServer.builder()
                .setHttpServerConfig(serverConfig)
                .setApplication(buildApplication(httpAddress, settings, secret, securityDomain))
                .build()) {
            server.bind();
            server.loop(); // does not return except by thread/signal interruption
        } catch (IOException e) {
            throw new IllegalStateException(e);
        }
    }

    public EventManager getEventManager() {
        return eventManager;
    }

    protected HttpAddress buildHttpAddress() {
        Settings settings = getSettings();
        String host = settings.get("host", "localhost");
        int port = settings.getAsInt("port", 8008);
        return HttpAddress.http1(host, port);
    }

    private Application buildApplication(HttpAddress httpAddress,
                                         Settings settings,
                                         String secret,
                                         HttpSecurityDomain securityDomain) throws IOException {
        for (InterlibraryServerModule interlibraryServerModule : ServiceLoader.load(InterlibraryServerModule.class)) {
            interlibraryServerModules.add(interlibraryServerModule);
        }
        logger.log(Level.INFO, "found interlibrary server modules " + interlibraryServerModules);
        this.eventManager = buildEventManager();
        Application application = InterlibraryServerApplication.builder()
                .setSettings(settings)
                .setSecret(secret)
                .setSecurityDomain(securityDomain)
                .setExecutor(BaseExecutor.builder()
                        .build())
                .setRouter(buildRouter(httpAddress, securityDomain))
                .build();
        ApplicationModule applicationModule = buildApplicationModule(settings);
        if (applicationModule != null) {
            application.addModule(applicationModule);
        }
        for (InterlibraryServerModule interlibraryServerModule : interlibraryServerModules) {
            application.addModule(interlibraryServerModule);
        }
        customizeApplication(application);
        return application;
    }

    protected EventManager buildEventManager() {
        EventManager.EventManagerBuilder builder = EventManager.builder();
        builder.setSettings(getSettings());
        for (InterlibraryServerModule interlibraryServerModule : interlibraryServerModules) {
            interlibraryServerModule.installEventServices(builder);
        }
        return builder.build();
    }

    protected HttpSecurityDomain buildSecurityDomain() {
        // you should override this for having a security domain
        return null;
    }

    private HttpRouter buildRouter(HttpAddress httpAddress,
                                   HttpSecurityDomain securityDomain) throws IOException {
        BaseHttpDomainBuilder domainBuilder = BaseHttpDomain.builder();
        domainBuilder.setHttpAddress(httpAddress);
        for (InterlibraryServerModule interlibraryServerModule : interlibraryServerModules) {
            interlibraryServerModule.installInto(this, securityDomain, domainBuilder);
        }
        BaseHttpRouterBuilder routerBuilder = BaseHttpRouter.builder();
        String prefix = Objects.requireNonNull(getSettings().get("web.prefix"), "web.prefix must not be null");
        routerBuilder.setPrefix(prefix);
        customizeRouter(routerBuilder);
        routerBuilder.addDomain(buildDomain(httpAddress, securityDomain, domainBuilder));
        return routerBuilder.build();
    }

    protected void customizeRouter(BaseHttpRouterBuilder routerBuilder) throws IOException {
    }

    protected HttpDomain buildDomain(HttpAddress httpAddress,
                                     HttpSecurityDomain securityDomain,
                                     BaseHttpDomainBuilder domainBuilder) throws IOException {
        return domainBuilder.build(); // dummy domain, without endpoints
    }

    protected ApplicationModule buildApplicationModule(Settings settings) {
        // you should override this
        return null;
    }

    protected void customizeApplication(Application application) {
    }

    protected static final String hexFavIcon =
            "000001000100101000000100200068040000160000002800000010000000" +
                    "200000000100200000000000000000000000000000000000000000000000" +
                    "000099330000993300009933000099330000993300009933000099330000" +
                    "993300009933000099330000993300009933000099330000993300009933" +
                    "00009933000099330000993300009933000099330000993300ff993300ff" +
                    "993300ff993300ff993300ff993300ff993300ff993300ff993300ff9933" +
                    "0000993300009933000099330000993300009933000099330000993300ff" +
                    "993300ff993300ff993300ff993300ff993300ff993300ff993300ff9933" +
                    "00ff99330000993300009933000099330000993300009933000099330000" +
                    "993300ff9933000099330000993300009933000099330000993300009933" +
                    "00009933000099330000993300009933000099330000993300ff993300ff" +
                    "99330000993300ff99330000993300ff993300ff993300ff993300ff9933" +
                    "00ff993300ff993300ff993300ff993300ff9933000099330000993300ff" +
                    "993300ff99330000993300ff99330000993300ff993300ff993300ff9933" +
                    "00ff993300ff993300ff993300ff993300ff993300ff9933000000000000" +
                    "993300ff993300ff00000000993300ff0000000000000000000000000000" +
                    "00000000000000000000000000000000000099330000993300ff99330000" +
                    "00000000993300ff993300ff00000000993300ff00000000000000000000" +
                    "0000000000000000000000000000993300ff993300ff00000000993300ff" +
                    "0000000000000000993300ff993300ff00000000993300ff993300009933" +
                    "000099330000993300009933000099330000993300ff993300ff99330000" +
                    "993300ff9933000000000000993300ff993300ff00000000993300ff9933" +
                    "00009933000099330000993300009933000099330000993300ff993300ff" +
                    "99330000993300ff9933000000000000993300ff993300ff000000009933" +
                    "00ff993300009933000099330000993300009933000099330000993300ff" +
                    "993300ff99330000993300ff9933000000000000993300ff993300ff0000" +
                    "000099330000993300009933000099330000993300009933000099330000" +
                    "993300ff993300ff99330000993300ff9933000000000000993300ff9933" +
                    "00ff00000000993300ff993300ff993300ff993300ff993300ff993300ff" +
                    "993300ff993300ff993300ff99330000993300ff99330000000000009933" +
                    "00ff993300ff000000009933000099330000993300009933000099330000" +
                    "993300009933000099330000993300009933000099330000993300000000" +
                    "0000993300ff993300ff993300ff993300ff993300ff993300ff993300ff" +
                    "993300ff993300ff993300ff993300ff993300ff99330000993300009933" +
                    "000000000000000000000000000000000000993300009933000099330000" +
                    "993300009933000099330000993300009933000099330000993300009933" +
                    "000099330000ffff0000f0070000f0070000f7ff00009401000094010000" +
                    "97fd000097e5000097e5000097e5000097e500009fe50000900500009fff" +
                    "000080070000ffff0000";


    protected static byte[] fromHex(String hex) {
        Objects.requireNonNull(hex);
        int len = hex.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) fromHex(hex.charAt(i), hex.charAt(i + 1));
        }
        return data;
    }

    private static int fromHex(int b1, int b2) {
        int i1 = Character.digit(b1, 16);
        if (i1 == -1) {
            throw new IllegalArgumentException("invalid character in hexadecimal: " + b1);
        }
        int i2 = Character.digit(b2, 16);
        if (i2 == -1) {
            throw new IllegalArgumentException("invalid character in hexadecimal: " + b2);
        }
        return (i1 << 4) + i2;
    }
}
