package org.xbib.interlibrary.http.server;

import java.nio.file.Paths;
import java.time.Duration;
import org.xbib.datastructures.api.TimeValue;
import org.xbib.net.UserProfile;
import org.xbib.net.http.cookie.SameSite;
import org.xbib.net.http.server.HttpHandler;
import org.xbib.net.http.server.application.BaseApplication;
import org.xbib.net.http.server.auth.FileJsonUserProfileCodec;
import org.xbib.net.http.server.persist.Codec;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.FileJsonSessionCodec;
import org.xbib.net.http.server.session.IncomingContextHandler;
import org.xbib.net.http.server.session.OutgoingContextHandler;
import org.xbib.net.http.server.session.Session;
import org.xbib.net.util.RandomUtil;

public class InterlibraryServerApplication extends BaseApplication {

    private Codec<UserProfile> userProfileCodec;

    private Codec<Session> sessionCodec;

    protected InterlibraryServerApplication(InterlibraryServerApplicationBuilder builder) {
        super(builder);
    }

    public static InterlibraryServerApplicationBuilder builder() {
        try {
            return new InterlibraryServerApplicationBuilder();
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    @Override
    protected Codec<UserProfile> newUserProfileCodec(HttpRouterContext httpRouterContext) {
        if (userProfileCodec == null) {
            String sessionDir = getSettings().get("userprofile.path", "/var/tmp/userprofile");
            this.userProfileCodec = new FileJsonUserProfileCodec(Paths.get(sessionDir));
        }
        return userProfileCodec;
    }

    @Override
    protected Codec<Session> newSessionCodec(HttpRouterContext httpRouterContext) {
        if (sessionCodec == null) {
            String sessionDir = getSettings().get("session.path", "/var/tmp/session");
            this.sessionCodec = new FileJsonSessionCodec(sessionName, this, 1024, Duration.ofDays(1),
                    Paths.get(sessionDir));
        }
        return sessionCodec;
    }

    @Override
    protected HttpHandler newIncomingContextHandler(Codec<UserProfile> userProfileCodec,
                                                    Codec<Session> sessionCodec) {
        String sessionCookieAlgorithm = getSettings().get("session.cookie.algo", "HmacSHA1");
        int sessionCookieIdLength = getSettings().getAsInt("session.idlen", 16);
        return new IncomingContextHandler(userProfileCodec,
                getSecret(),
                sessionCookieAlgorithm,
                sessionName,
                sessionCodec,
                getStaticFileSuffixes(),
                () -> RandomUtil.randomString(sessionCookieIdLength));
    }

    @Override
    protected OutgoingContextHandler newOutgoingContextHandler() {
        String sessionCookieAlgorithm = getSettings().get("session.cookie.algo", "HmacSHA1");
        TimeValue sessionDuration = getSettings().getAsTime("session.duration", TimeValue.timeValueHours(24));
        boolean httpOnly = getSettings().getAsBoolean("session.cookie.httponly", true);
        boolean secure = getSettings().getAsBoolean("session.cookie.secure", false);
        SameSite sameSite = SameSite.valueOf(getSettings().get("session.cookie.samesite", "LAX"));
        return new OutgoingContextHandler(
                getSecret(),
                sessionCookieAlgorithm,
                sessionName,
                getStaticFileSuffixes(),
                Duration.ofHours(sessionDuration.getHours()),
                httpOnly,
                secure,
                sameSite
        );
    }
}
